/*
        Copyright (c) 2014 by Plexim GmbH
        All rights reserved.

        A free license is granted to anyone to use this software for any legal
        non safety-critical purpose, including commercial applications, provided
        that:
        1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
        2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
        OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
        FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
        AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
        LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
        OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
        SOFTWARE.
 */

#include "init.h"
#include "uart.h"
#include "control.h"

void InitClock ( void )
{
#if 0
    // GS302 - Power converter part

    // Configure Oscillator to operate the device at 40 MIPS
    //   Fosc = Fin*M/(N1*N2), Fcy = Fosc/2
    //   Fosc = 7.37*(43)/(2*2) = 80MHz for Fosc, Fcy = 40MHz

    // Configure PLL prescaler, PLL postscaler, PLL divisor
    PLLFBD = 41 ;                // M = PLLFBD + 2
    CLKDIVbits.PLLPOST = 0 ;     // N1 = 2
    CLKDIVbits.PLLPRE = 0 ;      // N2 = 2

    // Change oscillator to FRC + PLL
    __builtin_write_OSCCONH ( 0x01 ) ;	// New Oscillator FRC w/ PLL
    __builtin_write_OSCCONL ( 0x01 ) ;  	// Enable Clock Switch

    while (OSCCONbits.COSC != 0b001) ;	// Wait for new Oscillator - FRC w/ PLL
    while (OSCCONbits.LOCK != 1) ;	// Wait for Pll to Lock

    // Now setup the ADC and PWM clock for 120MHz
    //   ((FRC * 16) / APSTSCLR ) = (7.37 * 16) / 1 = ~ 120MHz

    ACLKCONbits.FRCSEL = 1 ;	// FRC provides input to Auxiliary PLL (x16)

    ACLKCONbits.SELACLK = 1 ;	// Auxiliary Ocillator provides clock
    // source for PWM & ADC
    ACLKCONbits.APSTSCLR = 7 ;   // Divide Auxiliary clock by 1
    ACLKCONbits.ENAPLL = 1 ;     // Enable Auxiliary PLL

    while (ACLKCONbits.APLLCK != 1) ;	// Wait for Aux. PLL to Lock
#else
    // MC802 - Motor Control part
    PLLFBD  =   41; 			/* M = PLLFBD + 2 */
    CLKDIVbits.PLLPOST  = 0;            /* N1 = 2 */
    CLKDIVbits.PLLPRE   = 0;            /* N2 = 2 */

    __builtin_write_OSCCONH(0x01);	/* New Oscillator FRC w/ PLL */
    __builtin_write_OSCCONL(0x01);  	/* Enable Switch */

    while(OSCCONbits.COSC != 0b001);	/* Wait for new Oscillator to become FRC w/ PLL */
    while(OSCCONbits.LOCK != 1);	/* Wait for Pll to Lock */
#endif
}

void InitIOPorts ( void )
{
    LATAbits.LATA0 = 0 ;       // LED on stick
    TRISAbits.TRISA0 = 0;      // Set as port A0 pins as output
}

void InitUart( void )
{
    unsigned int    U1MODEvalue;
    unsigned int    U1STAvalue;
    unsigned int    baudvalue;

    __builtin_write_OSCCONL(OSCCON & ~(1<<6));
     //configure PPS pins per DPSK pinout
    RPINR18bits.U1RXR = 11; // Assign UART1 reveice (U1RX) to correspond to RP 11
    RPOR5bits.RP10R = 3; // Make RP10 output pin a UART1 transmit pint (U1TX)
    __builtin_write_OSCCONL(OSCCON | (1<<6));

    CloseUART1();
    ConfigIntUART1(UART_RX_INT_DIS & UART_RX_INT_PR6 &
                   UART_TX_INT_DIS & UART_TX_INT_PR2);
    baudvalue =   UBRGVAL;
    U1MODEvalue = UART_EN & UART_IDLE_CON & UART_IrDA_DISABLE &
		  UART_MODE_FLOW & UART_UEN_00 & UART_DIS_WAKE &
		  UART_DIS_LOOPBACK & UART_DIS_ABAUD & UART_UXRX_IDLE_ONE &
	          UART_BRGH_FOUR & UART_NO_PAR_8BIT & UART_1STOPBIT;

    U1STAvalue  = UART_INT_TX_BUF_EMPTY  & UART_IrDA_POL_INV_ZERO &
	          UART_SYNC_BREAK_DISABLED & UART_TX_ENABLE & UART_INT_RX_CHAR &
	          UART_ADR_DETECT_DIS & UART_RX_OVERRUN_CLEAR;
    OpenUART1(U1MODEvalue, U1STAvalue, baudvalue);
}


// Need to Initialize the PWM here
void InitPWM(uint16_t aPRD)
{
    P1TCONbits.PTEN = 1;   //Enable PWM
    P1TCONbits.PTCKPS = 0;   //Set PWM time based clock prescalar to 1:1
    P1TCONbits.PTMOD = 2;   //Set PWM to operate in Up/Down mode with interrupts occuring once

    //Configure PWM1
    P1TPER = aPRD;   // F_pwm = Fcy / [(PxTPER+1)*2*1]

    PWM1CON1bits.PMOD1 = 0; //Operate the pin pair in Complimentary mode
    PWM1CON1bits.PEN1L = 1; //Operate PWM1 Low Pin as PWM pin
    PWM1CON1bits.PEN1H = 1; //Operate PWM1 High Pin as PWM pin

    PWM1CON2bits.IUE = 0;   //P1DC1 registers are synchronized to the PWM time base
    PWM1CON2bits.UDIS = 0;   //Duty cycle and period buffer registers are enabled

    //Configuring Special Event Trigger for ADC sampling
    PWM1CON2bits.SEVOPS = 0;   //Special event trigger postscalar is 1:1
    P1SECMPbits.SEVTCMP = 0;   // Special event trigger occurs when CTR is 0
    P1SECMPbits.SEVTDIR = 0;   //Trigger occurs in up counting mode
}

void InitDeadTimeModule()
{
    //Configuring DeadTime Unit A
    P1DTCON1bits.DTA = 10;
    P1DTCON1bits.DTAPS = 0;

    //Configuring DeadTime Unit B
    P1DTCON1bits.DTB = 10;
    P1DTCON1bits.DTBPS = 0;

    //Configuring Active and In-Active transition to use DeadTime Unit A
    P1DTCON2bits.DTS1A = 0;
    P1DTCON2bits.DTS1I = 0;
}
void InitADC()
{
    AD1CON1bits.AD12B = 0; //Operate as 4-channel 10-bit ADC
    AD1CON1bits.ASAM = 1;   //Sampling begins immediately after conversion
    AD1CON1bits.SIMSAM = 0;   //Multi-Channel sequential sampling
    AD1CON1bits.SSRC = 0b011;   //PWM1 ends sampling and starts conversion

    AD1CON2bits.ALTS = 0;   // Sample A always selected
    AD1CON2bits.BUFM = 0;   // Start filling buffer from start address
    AD1CON2bits.SMPI = 2;   // Generate interrupt every 3rd S/H operation
    AD1CON2bits.CHPS = 3;   // Convert all channels
    AD1CON2bits.CSCNA = 0;   //Do not scan CH0+ inputs
    AD1CON2bits.VCFG = 3;   // VRef+ and VRef- chosen

    AD1CON3bits.ADCS = 0;   // 1 Tcy
    AD1CON3bits.ADRC = 1;   //ADC Clock is derived from system clock

    AD1CHS0bits.CH0SA = 0; //Channel 0 positive input is AN0
    AD1CHS123bits.CH123SA = 1;   //Positive CH1 = AN3, CH2 = AN4, CH3 = AN 5

    AD1CHS0bits.CH0NA=0;   //Channel 0 negative input is VREFL
    AD1CHS123bits.CH123NA = 0;   //Negative CH1 = CH2 = CH3 = VREFL

    IFS0bits.AD1IF = 0;   //Clear the A/D interrupt flag
    

    // default offset calibration
    AIn.IuOffset = 0x200;
    AIn.IvOffset = 0x200;
}