/*
   Copyright (c) 2014 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#ifndef _F2806x_HALL_H_
#define _F2806x_HALL_H_

#include "pil.h"

//Initialization states for ECAP Control Registers 1 and 2 for ECAP1/2/3
#define HALL3_ECCTL2_INIT_STATE   ( CONTINUOUS_MODE + \
                                    TSCNTSTP_FREE + \
                                    SYNCI_DISABLE + \
                                    SYNCO_DISABLE + \
                                    CAPTURE_MODE )

//Define the structure of the Capture Driver Object
typedef struct
{
   uint16_t CmtnTrigHall;                   // Output: Commutation trigger for Mod6cnt input (0 or 0x7FFF)
   uint16_t CapCounter;                     // Variable: Running count of detected edges on ECAP1,2,3
   uint16_t DebounceCount;                  // Variable: Counter/debounce delay current value
   uint16_t DebounceAmount;                 // Parameter: Counter delay amount to validate/debounce GPIO readings

   //PIL Extension for probed variables
   PIL_OVERRIDE_PROBE(uint16_t, HallGpio, 0, 1, "");

   uint16_t HallGpioBuffer;                 // Variable: Buffer of last logic level on ECAP/GPIO while being debounced
   uint16_t HallGpioAccepted;               // Variable: Debounced logic level on ECAP/GPIO
   uint16_t EdgeDebounced;                  // Variable: Trigger from Debounce function to Hall_Drv, if = 0x7FFF edge is debounced
   uint16_t HallMap[6];                     // Variable: ECAP/GPIO logic levels for HallMapPointer = 0-5
   uint16_t CapFlag;                        // Variable: ECAP flags, indicating which ECAP detected the edge
   uint16_t StallCount;                     // Variable: If motor stalls, this counter overflow triggers
                                          //           commutation to start rotation. Rotation is defined as
                                          //           an edge detection of a hall signal.
   uint16_t HallMapPointer;                 // Input/Output (see note below): During hall map creation, this variable points to the
                                          //            current commutation state.  After map creation, it
                                          //            points to the next commutation state.
   int16_t Revolutions;                     // Parameter: Running counter, with a revolution defined as 1-cycle
                                          //            of the 6 hall states
} HALL3;

/* Note:

 - When HallMapPointer is as an input, it is defined by MOD6_CNT.

 - When HallMapPointer is as an output, during hall map creation,

 this variable points to the current commutation state.

 After map creation, it points to the next commutation state.

 */

//Default Initializer for the F2803X HALL3 Object
#define F2806XHALL3      {0,  \
                          0,  \
                          0,  \
                         10,  \
                          0,  \
                          0,  \
                          0,  \
                          0,  \
                          0,  \
                          0,  \
              {0,0,0,0,0,0},  \
                          0,  \
                     0xFFFF,  \
                          0,  \
                        -10,  \
                                  }

#define HALL3_DEFAULTS F2806XHALL3

extern HALL3 Hall1;

extern void initHall3(HALL3 *aV);
extern void determineHALL3State(HALL3 *aV);
#endif
