/*
   Copyright (c) 2014 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#ifndef CONTROL_H_
#define CONTROL_H_

#include "pil.h"

/* system FSM */
typedef enum
{
	SYSTEM_FSM_STATE_POWERUP,
	SYSTEM_FSM_STATE_CALIBRATE,
	SYSTEM_FSM_STATE_PRECHARGE_DC,
	SYSTEM_FSM_STATE_SYNCHRONIZE_AC,
	SYSTEM_FSM_STATE_READY,
	SYSTEM_FSM_STATE_ENABLED,
	SYSTEM_FSM_STATE_FAULT,
	SYSTEM_FSM_STATE_CRITICAL_FAULT,
	SYSTEM_FSM_STATE_FAULT_ACKN
} SystemFsmState_t;

typedef enum
{
	PS_FSM_STATE_POWERUP,
	PS_FSM_STATE_DISABLED,
	PS_FSM_STATE_ENABLING,
	PS_FSM_STATE_ENABLED,
	PS_FSM_STATE_FAULT,
	PS_FSM_STATE_CRITICAL_FAULT,
	PS_FSM_STATE_FAULT_ACKN
} PowerstageFsmState_t;


typedef struct SYS_FSM_VARS {
	int16_t state;
} SystemFsm_t;

typedef struct PS_FSM_VARS {
	int16_t state;
	uint16_t enable;
} PowerstageFsm_t;

typedef struct TASK1_VARS {
	PIL_READ_PROBE(uint16_t, stepCtr, 0, 1.0, "");
	PIL_READ_PROBE(uint16_t, Fs, 0, 1, "");
	PIL_OVERRIDE_PROBE(int16_t, Vsmax, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, IdSet, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, IqSet, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, we, PLX_PIDQ_Q_WE, W_REF, "");
	PIL_OVERRIDE_PROBE(int16_t, IdErr, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, IqErr, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, Ia, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, Ib, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, Id, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, Iq, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, Vd, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, Vq, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, VdFF, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, VqFF, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, VmidErr, Q_IV, VDC_REF, "V");
	PIL_READ_PROBE(int16_t, VdSat, Q_IV, VP_REF, "V");
	PIL_READ_PROBE(int16_t, VqSat, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, Va, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, Vb, Q_IV, VP_REF, "V");

	PIL_OVERRIDE_PROBE(uint16_t, redVector1Split, 15, 1, "");
	PIL_OVERRIDE_PROBE(uint16_t, redVector2Split, 15, 1, "");
	PIL_OVERRIDE_PROBE(int16_t, redVector2Adj, 15, 1, "");

	PIL_READ_PROBE(uint16_t, Cmp0T,  0, 1, "");
	PIL_READ_PROBE(uint16_t, Cmp1T,  0, 1, "");
	PIL_READ_PROBE(uint16_t, Cmp2T,  0, 1, "");

	PIL_READ_PROBE(uint16_t, Cmp0,  0, 1, "");
	PIL_READ_PROBE(uint16_t, Cmp1,  0, 1, "");
	PIL_READ_PROBE(uint16_t, Cmp2,  0, 1, "");
	PIL_READ_PROBE(uint16_t, Cmp3,  0, 1, "");
	PIL_READ_PROBE(uint16_t, Cmp4,  0, 1, "");
	PIL_READ_PROBE(uint16_t, Cmp5,  0, 1, "");

	PIL_READ_PROBE(uint16_t, psEnabled, 0, 1, "");
	PIL_READ_PROBE(uint16_t, psEnableReq, 0, 1, "");
	PIL_READ_PROBE(uint16_t, psDisableReq, 0, 1, "");

	PIL_OVERRIDE_PROBE(int16_t, svpwmLimitingGain, 14, 1, "");
	PIL_OVERRIDE_PROBE(uint16_t, fluxPosition, 16, 1, "rev");
	PIL_READ_PROBE(int16_t, fluxPosSin, 15, 1, "");
	PIL_READ_PROBE(int16_t, fluxPosCos, 15, 1, "");

	PIL_READ_PROBE(uint16_t, subHexagon, 0, 1, "");
	PIL_READ_PROBE(uint16_t, zone, 0, 1, "");
	PIL_READ_PROBE(int16_t, VaT, Q_IV, VP_REF, "V");
	PIL_READ_PROBE(int16_t, VbT, Q_IV, VP_REF, "V");

	PIL_OVERRIDE_PROBE(int16_t, VgridA, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, VgridB, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, Vgrid, Q_IV, VP_REF, "V");

	PIL_READ_PROBE(uint16_t, gridVoltageAngle, 16, 1, "rev");

	PIL_READ_PROBE(uint16_t, gridPllAngle, 16, 1, "rev");
	PIL_READ_PROBE(int16_t, gridPllOmega, 10, W_REF, "rad/s");
	PIL_OVERRIDE_PROBE(uint16_t, gridPllIsLocked, 0, 1, "");
} Task1Vars_t;

typedef struct TASK2_VARS {
	PIL_READ_PROBE(uint16_t, stepCtr, 0, 1.0, "");
	PIL_READ_PROBE(uint16_t, active, 0, 1.0, "");
	PIL_OVERRIDE_PROBE(int16_t, IdDesired, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, IqDesired, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, IdSet, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, IqSet, Q_IV, I_REF, "A");
	PIL_OVERRIDE_PROBE(int16_t, VdFF, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, VqFF, Q_IV, VP_REF, "V");
	PIL_OVERRIDE_PROBE(int16_t, thetaSms, 16, 1, "");

	SystemFsm_t sysFsm;
	PowerstageFsm_t psFsm;
} Task2Vars_t;

/* control tasks globals */
struct CONTROL_VARS {
	PIL_READ_PROBE(uint16_t, backgroundTaskNumber, 0, 1.0, "");

	Task1Vars_t tsk1;
	Task2Vars_t tsk2;
};

extern struct CONTROL_VARS Ctrl;

extern void InitControl(void);
extern void DisableActuation(void);
extern void EnableActuation(void);
extern void ResetControl(void);

extern void ControlTask1(void);
extern void ControlTask2(void);
extern void ControlBackground(void);


#endif /*CONTROL_H_*/
